%Perceptron je neuronov s tvoen jedinm neuronem, kter doke klasifikovat
%objekty (vstupn data) do dvou kategori.
%Tento program provd uen a nsledn pouit perceptronu pro rozdlen bod
%v rovin pmkou, na body "nad" a "pod" pmkou.

%Matematick operace
%%%%%%%%%%%%%%%%%%%%
  %Skalrn souin vektor A a B s pouitm akumultoru.
  dot_product_acc([],[],Acc,Acc).
  dot_product_acc([Ai|A],[Bi|B],Acc,Res) :- Acci is Acc + Ai * Bi, !, dot_product_acc(A,B,Acci,Res).
  dot_product(A,B,Res) :- dot_product_acc(A,B,0,Res).

  %Nsoben vektoru V skalrem C.
  scalar_mul(_,[],[]).
  scalar_mul(C,[Vi|V],[Ri|R]) :- Ri is Vi * C, scalar_mul(C,V,R).

  %Souet vektor A a B.
  vector_sum([],[],[]).
  vector_sum([Ai|A],[Bi|B],[Ri|R]) :- Ri is Ai + Bi, vector_sum(A,B,R).

%Klasifikace bodu
%%%%%%%%%%%%%%%%%
  %Zaad bod do tdy 1 nebo -1 podle jeho souadnic a nastaven vhy.
  %Je vhodn pout Class is X namsto doplnn do hlaviky, jinak je nco jinho +1, 1, 01 apod.
  classify(Point, Weights, Class) :- Class is +1, dot_product(Point,Weights,Res), Res >= 0.
  classify(_Point, _Weights, Class) :- Class is -1.

  %Zaad vechny body v seznamu do td.
  classify_all([],_,[]).
  classify_all([Point|OtherPoints],Weights,[Class|OtherClasses]) :-
      classify(Point,Weights,Class), !,
      classify_all(OtherPoints,Weights,OtherClasses).

%Uen perceptronu
%%%%%%%%%%%%%%%%%%
  %Jedna iterace uen - zpracovn jednoho bodu a poadovan tdy.
  transform(
    %Vstupn data - ze seznamu uebnch dat vybrna dal na ad
    in(LearnRate, [Point|OtherPoints], [Desired|OtherDesired], Weights, Count),
    %Vstupem jsou vstupn data pro dal iteraci
    in(LearnRate, NewPoints, NewDesired, NewWeights, NewCount)
  ) :-
      %Zaazen uebnch dat na konec seznamu pro opakovan oen: 
      append(OtherPoints,[Point],NewPoints),
      append(OtherDesired,[Desired],NewDesired),
      %Pouit dat k uen perceptronu:
      perceptron(LearnRate,Point,Desired,Weights,NewWeights),
      %Pipoten dal iterace:
      NewCount is Count + 1.

  %Uprav vhy perceptronu, pokud bod nedv do sprvn tdy.
  perceptron(LearnRate, Point, Desired, Weights, NewWeights) :-
      classify(Point, Weights, Class),
      Delta is LearnRate * (Desired - Class), %Li se tda?
      scalar_mul(Delta,Point,DeltaWeights), %Vypote zmnu vhy
      vector_sum(Weights,DeltaWeights,NewWeights).

  %U perceptron v danm potu iterac.
  %Argumenty jsou zde seskupeny do sloench term pro pehlednost a abychom
  %je nemuseli vypisovat, pokud danou skupinu k niemu nepouvme.
  pta(in(_LearnRate, Points, Desired, Weights, IterCnt), out(Weights, IterCnt)) :-
      classify_all(Points,Weights,Desired), !. %Ukonovac podmnka
  pta(Arg, Result) :-
      transform(Arg,NewArg), !, pta(NewArg,Result). %Dal iterace uen

  %Sputn uen
  pta(LearnRate, Points, DesiredOutputs, Weights, FinalWeights, Iterations) :-
      pta(
        in(LearnRate,Points,DesiredOutputs,Weights,0),
        out(FinalWeights,Iterations)
      ).
  

%Zkuebn vstupn data:
  %Body
  points([[6.981,0.554,1],[14.414,4.466,1],[2.337,4.040,1],[8.500,4.496,1],[9.190,2.000,1],[1.149,6.100,1],[14.786,2.179,1],[7.842,6.331,1]]).
  %Poadovan tdy
  desired([-1,1,-1,1,-1,-1,1,1]).
  %Poten vhy
  weights([-0.51,-0.35,0.13]).

%Test klasifikace ped a po uen:
:- points(Points), desired(Desired), weights(DefWeights),
   classify_all(Points, DefWeights, DefClasses), %Zaazen do td nenauenm perceptronem
   write(DefClasses), nl,
   pta(0.25, Points, Desired, DefWeights, Weights, 800), %Uen osmisty iteracemi
   write(Weights), nl,
   classify_all(Points, Weights, Classes), %Zaazen do td nauenm perceptronem
   write(Classes), nl.
