%Odebrn prvnho prvku ze seznamu:
test1 :-
  L=[a,b,c],
  L=[_|M],
  write(M).

%Pidn prvku na zatek seznamu:
test2 :-
  M=[b,c],
  L=[a|M],
  write(L).

%V prologu je vestavn predikt "append" pro spojen dvou seznam.
%(Vtina tchto vestavnch predikt sice nen v ISO, ale norma
%Prologu je pomrn slab a minimln "append" a "member" najdeme
%prakticky vude.)
test3 :-
  append([a,b,c],[d,e],L),
  write(L).

%Tento predikt lze pout i opan, k nalezen rznch monost
%sestaven danho seznamu.
%Predikty "write(X),fail." zajist vypsn vech monch hodnot X -
%fail na konci zpsob, e pravidlo nebude nikdy splnno, tud Prolog
%zkou vechny monosti, a nakonec odpov "no".
test4 :-
  append(M,N,[a,b,c]),
  write(M),write('; '),write(N),nl,fail.

%Implementace prediktu nen sloit, meme si ho napsat i sami:
pripoj([],L,L). %Algoritmus vezme druh seznam jako vsledn
pripoj([H1|T1],L2,[H1|L3]) :- pripoj(T1,L2,L3). %Prvky z prvnho seznamu
                                                %postupn pid na zatek
%A vyzkouet, e funguje stejn:
test5 :-
  write('Spojeni: '),nl,
  pripoj([a,b,c],[d,e],L),write(L),nl,
  write('Moznosti: '),nl,
  pripoj(M,N,[a,b,c]),write(M),write('; '),write(N),nl,fail.

%B-Prolog nabz i "append" spojujc 3 seznamy (tady u nm
%mnostv kombinac roste pomrn rychle):
test6 :-
  append(L,M,N,[a,b,c,d,e]),
  write(L),write('; '),write(M),write('; '),write(N),nl,fail. %6! monost

%Dal vestavn predikt "member" zjiuje, zda je term v seznamu.
test7 :-
  member(b,[a,b,c]).
  
%Pochopiteln i ten lze pout tak pro prchod seznamem:
test8 :-
  member(X,[a,b,c]),
  write(X),write(' '),fail.

%Pokud je term v seznamu vcekrt, bude vcekrt vypsn:
test9 :-
  member(X,[a,b,b,a,b,b,a,b,c,a,b,d]),
  write(X),write(' '),fail.

%I tento predikt nen sloit implementovat:
%Jsou pouity anonymn promnn, co nen nutn pro funknost,
%ale zamezuje varovn kompiltoru "singleton variables".
obsahuje(_,[]) :- fail. %V przdnm seznamu nic nen
obsahuje(X,[X|_]) :- true. %Pokud je prvek na prvnm mst, je urit v seznamu
obsahuje(X,[_|L]) :- obsahuje(X,L). %Pokud se prvn prvek li, zkusme dal
%A ovit nai implementaci:
test10 :-
  obsahuje(b,[a,b,c]),
  obsahuje(X,[a,b,c]),write(X),nl,fail.

%Predikt "delete" odstran vechny vskyty termu ze seznamu:
test11 :-
  delete([a,b,b,a,b,b,a,b,c,a,b,d],b,L),
  write(L).

%Tento predikt zejm nelze pout opan, protoe 'b' lze do seznamu pidvat
%do nekonena (vsledkem je "no").
test11 :-
  delete(L,b,[a,a,a,c,a,d]),
  write(L),nl,fail.

%Toto pravidlo bude splnno a X bude navzno na "a".
test12 :-
  delete([a,b,c],X,[b,c]),
  write(X).
  
%Toto pravidlo splnno nebude.
test13 :-
  delete([a,b,c],X,[b]).

%Vlastn implementace odstraovn ze seznamu:
odstran(L,_,_) :- var(L), !, fail. %Zabrnn nekonenmu rozvoji
odstran([],_,[]) :- true.
odstran([H|T],H,L) :- odstran(T,H,L).
odstran([H|T],X,[H|L]) :- odstran(T,X,L), (not(var(X)); fail).
test14 :-
  odstran([a,b,b,a,b,b,a,b,c,a,b,d],b,L),
  write(L).

test15 :-
  odstran(L,b,[a,a,c,c]),
  write(L),nl,fail.

test16 :-
  odstran([a,b,c],X,[b,c]),
  write(X).

%Pokud porovnme vestavn a vlastn predikt pro odstraovn s tm, e zadme
%pouze pvodn seznam, zjistme, e vestavn predikt vype pouze prvn
%monost, zatmco n vlastn vechny. Meme tedy tvrdit, e je lep,
%na druhou stranu, takov chovn mon nikdy nevyuijeme a je pomalej.
test17 :-
  (
    write('delete([a,b,c],X,L)'), nl,
    delete([a,b,c],X,L),
    write(X), write('|'), write(L), nl, fail
  ); (
    write('odstran([a,b,c],X,L)'), nl,
    odstran([a,b,c],X,L),
    write(X), write('|'), write(L), nl, fail
  ).

%Odzne zbytek seznamu, pokud jde o volnou promnnou.
%Neodstran vak volnou promnnou, kter je poslednm prvkem seznamu.
test18 :-
  L = [a,b,c|Volny_zbytek],
  write('Pred oriznutim: '), write(L), nl,
  closetail(L),
  write('Po oriznuti: '), write(L), nl,
  M = [a,b,c,Volny_prvek], closetail(M),
  write('Volna promenna jako prvek: '), write(M).

%Zjist, zda N-t argument struktury odpovd termu. Pouze 3 argument sm bt voln
%promnn, aby program el zkompilovat, a v takovm ppad je pravidlo vdy
%splnno.
test19 :-
  arg(2,pismenka(a,b,c,d),b),
  arg(-87,pismenka(a,b,c,d),X).

%Stejn funkce, jako pedchoz, ale pracuje se seznamy.
test20 :-
  nth0(1,[a,b,c,d],b), %Varianta indexujc od nuly
  nth1(2,[a,b,c,d],b), %Varianta indexujc od jedniky
  nth(2,[a,b,c,d],b), %Bez upesnn indexuje od jedniky
  nth(2,[a,b,c,d],X), write('Druhy prvek: '), write(X). %vype "b"

%Lze najt vechny vskyty termu (indexy).
test21 :-
  write('Indexy b:'), nth(I,[a,b,b,a,b,b,a,b,c,a,b,d],b), write(' '), write(I), fail.

%Lze tak vytvoit seznam, kde bude term na danm indexu (ped nm budou voln
%promnn, zbytek (tail) seznamu bude tak voln promnn).
test22 :-
  nth(2,L,b), write(L).

%Predikt functor zjist, zda dan predikt m funktor danho jmna a danou aritu.
%Pokud je prvn argument voln promnn, predikt je vdy splnn.
test23 :-
  functor(pismenka(a,b,c,d),pismenka,4).

%Pokud je jeden z argument voln promnn, predikt membchk nen nikdy splnn.
test24 :-
  membchk(c,[a,b,c]).

%Tdn seznamu
test25 :-
  sort(<,[6,2,4,3,1,5],Setrideny),
  write(Setrideny), nl,
  sort(<,[b,e,c,f,a,d],Abecedni),
  write(Abecedni), nl,
  sort(<,['hotdog s kecupem','slanina s maslem','hamburgr s hranolkama','sirup bez ledu'],Jidla),
  write(Jidla), nl,
  sort(<,[[3,5,2],[1,4,1],[3,5,7],[1,3,2]],Posloupnosti),
  write(Posloupnosti).

%Pedn prediktu jako agrument, pouit call/n:
secti(A,B,V) :- V is A + B.
vynasob(A,B,V) :- V is A * B.
maximum(A,B,V) :- V is max(A,B).
minimum(A,B,V) :- V is min(A,B).
zpracuj(_,[H|[]],V) :- V is H.
zpracuj(Operator,[H|T],V2) :- zpracuj(Operator,T,V1), call(Operator,V1,H,V2).
test26 :-
  Cisla = [1,2,3,4,5,6],
  zpracuj(secti,Cisla,Vysledek1), write('Soucet: '), write(Vysledek1), nl,
  zpracuj(vynasob,Cisla,Vysledek2), write('Soucin: '), write(Vysledek2), nl,
  zpracuj(maximum,Cisla,Vysledek3), write('Maximum: '), write(Vysledek3), nl,
  zpracuj(minimum,Cisla,Vysledek4), write('Minimum: '), write(Vysledek4), nl.
